indexing

   title:       "Supplies some additional string service";
   cluster:     "format/science [format/science]";
   project:     "The Universal Simple Formatting Library (USFL)";
   copyright:   "SIG Computer, 1995";
   author:      "Andy Malakov  (andy@eiffel.ru) and",
                "Igor Nesterof (nest@eiffel.ru)";
   original:    08,Sep,1995;
   version:     1.0;
   last_change: 14,Sep,1995;
   done_at:     "SIG Computer (info@eiffel.ru)";
   extrnl_name: "ext_str.e"

class STRING_TOOLS

   feature
   ------------------------------------------------------------ make_with_fill
      fill_string ( nr_blanks : INTEGER, pc : CHARACTER ) : STRING is
         do
            !!Result.make(nr_blanks)
            Result.fill_with(pc)
         end
   ------------------------------------------------------------left_pad
      left_pad(s: STRING,pc : CHARACTER, l : INTEGER) : STRING is
         local
            sl  : INTEGER
         do
            sl := s.count
            if sl >=l then
               !!Result.adapt(s)
            else
               !!Result.adapt(fill_string(l-sl,pc))
               Result.append(s)
            end
         end
   ------------------------------------------------------------right_pad
      right_pad(s : STRING, pc : CHARACTER, l : INTEGER) : STRING is
         do
            !!Result.adapt(s)
            if s.count < l then
               Result.append(fill_string(l-s.count,pc))
            end
         end
   ------------------------------------------------------------center_pad
      center_pad(s: STRING,pc : CHARACTER, l : INTEGER) : STRING is
         local
            sl, half  : INTEGER

         do
            sl := s.count
            if sl >=l then
               !!Result.adapt(s)
            else
               half := (l-sl)//2
               !!Result.adapt(fill_string(half,pc))
               Result.append(s)
               Result.append(fill_string(l-sl-half,pc))
            end
         end
   ------------------------------------------------------------pad_string
      pad_string(s : STRING, pc : CHARACTER, l : INTEGER, from_left : BOOLEAN) : STRING is
         do
            if from_left then
               !!Result.adapt(left_pad(s,pc,l))
            else
               !!Result.adapt(right_pad(s,pc,l))
            end
         end
      ------------------------------------------------------------cut_here
      cut_here(s : STRING, fin_len : INTEGER) : STRING is
      -- make string not more then fin_len characters
      require
         positive_len : fin_len >=0
         do
            if s.count > fin_len then
               !!Result.adapt(x_substring(s,1,fin_len))
            else
               !!Result.adapt(s)
            end
         end
      ------------------------------------------------------------ strip_zeros
      -- strip trailing 0's
      -- pretty functional programming style :-)
      -- I'm too lazy to rewrite it in iteration style
      strip_zeros(s : STRING) : STRING is
      local
         do
            if s.is_equal("") then
               !!Result.adapt(s)
            elseif s.item(s.count) = '0' then
               !!Result.adapt(strip_zeros(s.substring(1,s.count-1)))
            else
               !!Result.adapt(s)
            end
         end
      --------------------------------------------------------
      ifc( b : BOOLEAN, t,f : CHARACTER) : CHARACTER is
      -- C-like style ? : operator
         do
            if b then
               Result := t
            else
               Result := f
            end
         end
   ----------------------------------------------------------------------
      is_digit ( s : STRING , indx : INTEGER ) : BOOLEAN is
         local c : CHARACTER
         do
            c := s.item ( indx )
            if c >= '0' and then c <= '9' then
               Result := true
            end
         end
   -------------------------------------------------------------------
      is_letter ( s : STRING ,indx : INTEGER ) : BOOLEAN is
         local c : CHARACTER
         do
            c := s.item ( indx )
            if ( c >= 'a' and then c <= 'z') or else
               ( c >= 'A' and then c <= 'Z') then
               Result := true
            end
         end
   -------------------------------------------------------------------
      index ( s : STRING ,pc: CHARACTER ) : INTEGER is
            -- return pc position or zero if it not found
      require
         s /= Void
         local
            i : INTEGER
         do
            from i := 1 until i > s.count loop
               if s.item (i) = pc then
                  Result := i
                  i := s.count + 1
               else
                  i := i + 1
               end
            end
         end
   -------------------------------------------------------------------
      contain ( s : STRING ,pc: CHARACTER ) : BOOLEAN is
         do
            Result := index (s, pc) /= 0
         end
   ----------------------------------------------------------------------
      x_substring
         ( s : STRING, from_index:INTEGER, to_index:INTEGER) : STRING is
            -- extend substring with empty string
      require
         sub_interval : 1 <= from_index and then
                        from_index <= to_index + 1 and then
                        to_index <= s.count
         do
            if from_index = to_index + 1 then
               Result := ""
            else
               Result := s.substring (from_index, to_index)
            end
         ensure
            Result /= Void
         end -- x_substring
   ----------------------------------------------------------------------
      find_itself ( s : STRING; list : ARRAY [ STRING ] ) : INTEGER is
         -- find s in strings list
         local
            i : INTEGER
            found : BOOLEAN
         do
            from i := list.lower
            until found or else i > list.upper
            loop
               if s.is_equal ( list @ i ) then
                  found := true
               else
                  i := i + 1
               end
            end
            if found then
               Result := i
            else
               Result := list.lower - 1
            end
         end -- find_itself
   ----------------------------------------------------------------------
      Zero : CHARACTER is '0';
   ----------------------------------------------------------------------
      x_s2i ( s : STRING ) : INTEGER is
            -- convert without checking
         require
            s /= Void
         local
            i : INTEGER
         do
            from
               i := 1
            until
               i > s.count or else not is_digit (s, i)
            loop
               Result := Result * 10 + s.item(i).to_integer - Zero.to_integer
               i := i + 1
            end
         end
   ----------------------------------------------------------------------
   -- Portability STUBS
   ----------------------------------------------------------------------
      x_append ( s, other : STRING ) is
         do
            s.append ( other )
         end

   ----------------------------------------------------------------------
      x_extend ( s : STRING; c : CHARACTER ) is
         do
            s.extend ( c )
         end
   ----------------------------------------------------------------------
      x_adapt ( s, other : STRING ) is
         do
            s.adapt ( other )
         end
   ----------------------------------------------------------------------
      x_prepend ( s, other : STRING ) is
         do
            s.prepend ( other )
         end

end -- class STRING_TOOLS
